#include <qapplication.h>
#include <qcolor.h>
#include <qkeycode.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qwidget.h>

/**
  * A class that lets the user draw scribbles with the mouse. The
  * window knows how to redraw itself.
  */
class ScribbleWindow : public QWidget
{
public:
  ScribbleWindow();

protected:
  virtual void mousePressEvent( QMouseEvent* );
  virtual void mouseMoveEvent( QMouseEvent* );
  virtual void paintEvent( QPaintEvent* );
  virtual void resizeEvent( QResizeEvent* );
  virtual void keyPressEvent( QKeyEvent* );

private:
  QPoint _last;

  QPixmap _buffer;
  QColor _color;
};

/** The constructor. Rather simple for now. */
ScribbleWindow::ScribbleWindow()
{
  // don't blank the window before repainting
  setBackgroundMode( NoBackground );

  // set an initial color
  _color = black;
}


/**
  * This virtual method gets called whenever the users presses the
  * mouse over the window. It just records the position of the mouse
  * at the time of the click.
  */
void ScribbleWindow::mousePressEvent( QMouseEvent* event )
{
  _last = event->pos(); // retrieve the coordinates from the event
}


/**
  * This virtual method gets called whenever the user moves the mouse
  * while the mouse button is pressed (this is also known as
  * "dragging"). If we had called setMouseTracking( true ) before,
  * this method would also be called when the mouse was moved without
  * any button pressed. We know that we haven't, and thus don't have
  * to check whether any buttons are pressed. One caveat: If you use
  * tooltips, mouse tracking is turned on automatically.
  */
void ScribbleWindow::mouseMoveEvent( QMouseEvent* event )
{
  // create a QPainter object for drawing onto the window
  QPainter windowpainter;
  // and another QPainter object for drawing into an off-screen pixmap
  QPainter bufferpainter;

  // start painting
  windowpainter.begin( this );    // this painter paints onto the window
  bufferpainter.begin( &_buffer ); // and this one in the buffer

  // set the color
  windowpainter.setPen( _color );
  bufferpainter.setPen( _color );

  // draw a line in both the window and the buffer
  windowpainter.drawLine( _last, event->pos() );
  bufferpainter.drawLine( _last, event->pos() );

  // done with painting
  windowpainter.end();
  bufferpainter.end();

  // remember the current mouse position
  _last = event->pos();
}



/** 
  *	This virtual method gets called whenever the widget needs
  * painting, e.g., when it has been obscured and unhidden again.
  */
void ScribbleWindow::paintEvent( QPaintEvent* event )
{
  // copy the image from the buffer pixmap to the window
  bitBlt( this, 0, 0, &_buffer );
}


/**
  * This virtual method gets called whenever the window is resized. We
  * use it to make sure that the off-screen buffer always has the same
  * size as the window.
  * In order not to lose the original scribbling, it is first copied
  * to a temporary buffer. After the main buffer has been resized and
  * filled with white, the image is copied from the temporary buffer to
  * the main buffer.
  */
void ScribbleWindow::resizeEvent( QResizeEvent* event )
{
  QPixmap save( _buffer );
  _buffer.resize( event->size() );
  _buffer.fill( white );
  bitBlt( &_buffer, 0, 0, &save );
}


/**
 * This virtual method is called whenever a key is pressed in the
 * window. It changes the current drawing color.
 */
void ScribbleWindow::keyPressEvent( QKeyEvent* event )
{
  switch( event->key() ) {
  case Key_F1:
	_color = black;
	break;
  case Key_F2:
	_color = darkGray;
	break;
  case Key_F3:
	_color = gray;
	break;
  case Key_F4:
	_color = lightGray;
	break;
  case Key_F5:
	_color = red;
	break;
  case Key_F6:
	_color = green;
	break;
  case Key_F7:
	_color = blue;
	break;
  case Key_F8:
	_color = cyan;
	break;
  case Key_F9:
	_color = magenta;
	break;
  case Key_F10:
	_color = yellow;
	break;
  case Key_F11:
	_color = darkRed;
	break;
  case Key_F12:
	_color = darkGreen;
	break;
  default:
	_color = darkBlue;
  }
}

int main( int argc, char* argv[] )
{
  QApplication myapp( argc, argv );

  ScribbleWindow* mywidget = new ScribbleWindow();
  mywidget->setGeometry( 50, 50, 400, 400 );
  
  myapp.setMainWidget( mywidget );
  mywidget->show();
  return myapp.exec();
}
