#include <qapplication.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qwidget.h>

/**
  * A class that lets the user draw scribbles with the mouse. The
  * window knows how to redraw itself.
  */
class ScribbleWindow : public QWidget
{
public:
  ScribbleWindow();

protected:
  virtual void mousePressEvent( QMouseEvent* );
  virtual void mouseMoveEvent( QMouseEvent* );
  virtual void paintEvent( QPaintEvent* );
  virtual void resizeEvent( QResizeEvent* );
  virtual void mouseDoubleClickEvent( QMouseEvent* );

private:
  QPoint _last;

  QPixmap _buffer;
};

/** The constructor. Rather simple for now. */
ScribbleWindow::ScribbleWindow()
{
  // don't blank the window before repainting
  setBackgroundMode( NoBackground );
}


/**
  * This virtual method gets called whenever the users presses the
  * mouse over the window. It just records the position of the mouse
  * at the time of the click.
  */
void ScribbleWindow::mousePressEvent( QMouseEvent* event )
{
  _last = event->pos(); // retrieve the coordinates from the event
}


/**
  * This virtual method gets called whenever the user moves the mouse
  * while the mouse button is pressed (this is also known as
  * "dragging"). If we had called setMouseTracking( true ) before,
  * this method would also be called when the mouse was moved without
  * any button pressed. We know that we haven't, and thus don't have
  * to check whether any buttons are pressed. One caveat: If you use
  * tooltips, mouse tracking is turned on automatically.
  */
void ScribbleWindow::mouseMoveEvent( QMouseEvent* event )
{
  // create a QPainter object for drawing onto the window
  QPainter windowpainter;
  // and another QPainter object for drawing into an off-screen pixmap
  QPainter bufferpainter;

  // start painting
  windowpainter.begin( this );    // this painter paints onto the window
  bufferpainter.begin( &_buffer ); // and this one in the buffer

  // draw a line in both the window and the buffer
  windowpainter.drawLine( _last, event->pos() );
  bufferpainter.drawLine( _last, event->pos() );

  // done with painting
  windowpainter.end();
  bufferpainter.end();

  // remember the current mouse position
  _last = event->pos();
}


/**
 * This virtual method is called whenever the user double-clicks into
 * the scribble window. It closes the application
 */
void ScribbleWindow::mouseDoubleClickEvent( QMouseEvent* )
{
  qApp->quit();
}


/** 
  *	This virtual method gets called whenever the widget needs
  * painting, e.g., when it has been obscured and unhidden again.
  */
void ScribbleWindow::paintEvent( QPaintEvent* event )
{
  // copy the image from the buffer pixmap to the window
  bitBlt( this, 0, 0, &_buffer );
}


/**
  * This virtual method gets called whenever the window is resized. We
  * use it to make sure that the off-screen buffer always has the same
  * size as the window.
  * In order not to lose the original scribbling, it is first copied
  * to a temporary buffer. After the main buffer has been resized and
  * filled with white, the image is copied from the temporary buffer to
  * the main buffer.
  */
void ScribbleWindow::resizeEvent( QResizeEvent* event )
{
  QPixmap save( _buffer );
  _buffer.resize( event->size() );
  _buffer.fill( white );
  bitBlt( &_buffer, 0, 0, &save );
}


int main( int argc, char* argv[] )
{
  QApplication myapp( argc, argv );

  ScribbleWindow* mywidget = new ScribbleWindow();
  mywidget->setGeometry( 50, 50, 400, 400 );
  
  myapp.setMainWidget( mywidget );
  mywidget->show();
  return myapp.exec();
}
