/*
 * A simple mirroring HTTP server in Qt.
 *
 * Copyright 1998, Matthias Kalle Dalheimer
 */

#define PORT 8080

#include <qapplication.h>
#include <qfile.h>
#include <qintdict.h>
#include <qsocketnotifier.h>
#include <qstring.h>
#include <qtextstream.h>

/* Platform-specific includes. */
#ifdef _WS_X11_
#include <netinet/in.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <unistd.h>
#elif _WS_WIN_
#endif

class Socket
{
public:
  Socket();
  ~Socket();

  void close();
  int handle() { return _fd; }

  void read( QString& text );
  void write( const char* text );

  void setFd( int _fd );

private:
  int _fd;
  QFile* _file;
  QTextStream* _tstream;
};


class ServerSocket : public Socket
{
public:
  ServerSocket( int port );

  Socket* accept();
};


class HTTPMirrorServer : QObject
{
  Q_OBJECT

public:
  HTTPMirrorServer();
  ~HTTPMirrorServer();

private slots:
  void connectionWanted( int );
  void dataArrived( int );

private:
  ServerSocket* _serversocket;
  QSocketNotifier* _notifier;
  QIntDict<Socket> _socketlist;
  QIntDict<QSocketNotifier> _notifierlist;
};


#include "httpserv.moc"

/* The Socket constructor only initializes the member variables. */
Socket::Socket()
{
  _file = 0;
  _tstream = 0;
  _fd = -1;
}


/* Close the socket when destroyed in case this has not been done
   yet. Delete the QFile and the QTextStream. */
 
Socket::~Socket()
{
  close();

  delete _file;
  delete _tstream;    
}


/* Close the socket. */
void Socket::close()
{
  ::close( _fd );
  _file->close();
}


/* Sets the _fd member and creates QFile/QTextStream pair for easier I/O. */
void Socket::setFd( int newfd )
{
  _fd = newfd;

  // Close and delete the old _file and _tstream in case it had already
  // been opened. 
  delete _file;
  delete _tstream;

  // Open with the new file descriptor
  _file = new QFile();
  _file->open( IO_ReadWrite | IO_Raw, newfd );

  // Create a QTextStream object on that file
  _tstream = new QTextStream( _file );
}
  
  

/* Read one line of data from the socket. */
void Socket::read( QString& text )
{
  text = _tstream->readLine();
}


/* Write data to the socket. */
void Socket::write( const char* text )
{
  *_tstream << text;
}



/* Create a server socket that binds the port passed in the
   argument. After creation, the socket is already set up for
   accepting connections. */
ServerSocket::ServerSocket( int port )
{
  // create a OS-level socket
  int newfd = socket( AF_INET, SOCK_STREAM, 0 );
  
  struct sockaddr_in serverAddress;
  memset( &serverAddress, 0, sizeof( serverAddress ) );
  serverAddress.sin_family = AF_INET;
  serverAddress.sin_addr.s_addr = htonl( INADDR_ANY );
  serverAddress.sin_port = htons( port );
  
  // bind the socket to the port
  bind( newfd, (sockaddr*)&serverAddress, sizeof( serverAddress ) );

  // and listen (allow up to five backlogged connections)
  listen( newfd, 5 );

  setFd( newfd );
}


/* Accept a connection by creating a new socket for passing data
   between client and server. It is the caller's responsibility to
   delete the Socket object. */
Socket* ServerSocket::accept()
{
  struct sockaddr client_addr; /* will contain the client's address,
								  not used here */ 
  int len;
  // accept the connection
  int newfd = ::accept( handle(), &client_addr, &len );
  Socket* newSocket = new Socket();
  newSocket->setFd( newfd );

  return newSocket;
}




/* Constructs the server by creating a QKServerSocket on a
   (hard-coded) port.  */
HTTPMirrorServer::HTTPMirrorServer()
{
  // create the socket
  _serversocket = new ServerSocket( PORT );

  // demand notification when there is data to be read from the socket
  _notifier = new QSocketNotifier( _serversocket->handle(),
								  QSocketNotifier::Read );
  connect( _notifier, SIGNAL( activated( int ) ),
		   this, SLOT( connectionWanted( int ) ) );
}


/* Destroys the server by removing the notifier and the server
   socket. */
HTTPMirrorServer::~HTTPMirrorServer()
{
  delete _notifier;
  delete _serversocket;
}


/* This slot is called whenever data is coming from the server socket, 
   i.e., a client demands a connection. It accepts the connection and
   sets a socket that will do the actual communication.*/
void HTTPMirrorServer::connectionWanted( int )
{
  // Accept the connection. This automatically creates a new socket
  // for the communication.
  Socket* socket = _serversocket->accept();

  // Enter the socket/filedescriptor pair into a list so that we can
  // find the address of the socket later.
  _socketlist.insert( socket->handle(), socket );

  // Demand notification for this socket as well.
  QSocketNotifier* datanotifier = new QSocketNotifier( socket->handle(),
													   QSocketNotifier::Read );
  connect( datanotifier, SIGNAL( activated( int ) ),
		   this, SLOT( dataArrived( int ) ) );

  // Enter the notifier in a list as well so that it can be deleted
  // later on.
  _notifierlist.insert( socket->handle(), datanotifier );
}


/* This slot is called when data arrives via one of the data
   sockets. In reads the header (everything until an empty line) and
   echoes it back to the client. */
void HTTPMirrorServer::dataArrived( int handle )
{
  // Retrieve the Socket object that corresponds to the handle. after
  // this slot has run, we don't need the socket any longer, so we can 
  // now use take() to remove it from the list.
  Socket* socket = _socketlist.take( handle );
  
  if( !socket )
	// Something strange has happened, better back out of here.
	return;

  // Read the data line by line and echo it back until a blank line
  // (which marks the end of the header) occurs.
  QString line;
  socket->read( line );
  while( !line.isEmpty() )
	{
	  // Echo back
	  line += '\n';
	  socket->write( line );
	  // Get next line
	  socket->read( line );
	}

  // We do not need the socket any longer, therefore remove its
  // notifier from list and delete the socket itself and the
  // notifier. The socket has already been removed from the list a few 
  // lines up.
  delete socket;
  delete _notifierlist.take( handle );
}


int main( int argc, char* argv[] )
{
  // We need a QApplication object, but there won't be any widgets. 
  QApplication a( argc, argv );

  // Creating a server object starts the server.
  HTTPMirrorServer server;

  return a.exec();
}
