package ScribbleArea;

use QColor;
use QPixmap;
use QWidget;
use QPaintDevice;

@ISA = qw( QWidget );

use slots 'setColor(QColor)';

#
# The constructor. Initializes the member variables.
#
sub new {
  my $class = shift;
  my $self = $class->SUPER::new(@_);

  # initialize member variables
  $$self{'_last'} = new QPoint;
  $$self{'_currentcolor'} = $black;
  $$self{'_buffer'} = new QPixmap;
  
  # don't blank the window before repainting
  $self->setBackgroundMode( $BackgroundMode{NoBackground} );

  return $self;
}

#
# The destructor. Does nothing in this version.
#
sub DESTROY {
}


#
# This slot sets the current color for the scribble area. It will be
# connected with the colorChanged(QColor) signal from the
# ScribbleWindow. 
#
sub setColor {
  my $self = shift;
  $$self{'_currentcolor'} = shift;
}


#
# This virtual method is called whenever the user presses the mouse
# over the window. It just records the position of the mouse at the
# time of the click.
#
sub mousePressEvent {
  my $self = shift;
  my $event = shift;
  $$self{'_last'} = $event->pos();
}


#
# This virtual method gets called whenever the user moves the mouse
# while the mouse button is pressed (this is also known as
# "dragging"). If we had called setMouseTracking( true ) before,
# this method would also be called when the mouse was moved without
# any button pressed. We know that we haven't, and thus don't have
# to check whether any buttons are pressed. One caveat: If you use
# tooltips, mouse tracking is turned on automatically.
#
sub mouseMoveEvent {
  my $self = shift;
  my $event = shift;

  # create a QPainter object for drawing onto the window
  my $windowpainter = new QPainter;
  # and another QPainter object for drawing into an off-screen pixmap
  my $bufferpainter = new QPainter;

  # start painting
  $windowpainter->begin( $self ); # this painter paints onto the window 
  $bufferpainter->begin( $$self{'_buffer'} ); # and this one in the
                                             # buffer 
  
  # set a standard pen with the currently selected color
  $windowpainter->setPen( $$self{'_currentcolor'} );
  $bufferpainter->setPen( $$self{'_currentcolor'} );

  # draw a line in both the window and the buffer
  $windowpainter->drawLine( $$self{'_last'}, $event->pos() );
  $bufferpainter->drawLine( $$self{'_last'}, $event->pos() );

  # done with painting
  $windowpainter->end();
  $bufferpainter->end();

  # remember the current mouse position
  $$self{'_last'} = $event->pos();
}


#
# This virtual method is called whenever the widget needs painting,
# e.g., when it has been obscured and unhidden again.
#
sub paintEvent {
  my $self = shift;

  # copy the image from the buffer pixmap to the window
  my $buffer = $$self{'_buffer'};
  $self->bitBlt( 0, 0, $buffer );
}


#
# This virtual method gets called whenever the window is resized. We
# use it to make sure that the off-screen buffer always has the same
# size as the window.
# In order not to lose the original scribbling, it is first copied
# to a temporary buffer. After the main buffer has been resized and
# filled with white, the image is copied from the temporary buffer to
# the main buffer.
#
sub resizeEvent {
  my $self = shift;
  my $event = shift;

  my $buffer = $$self{'_buffer'};
  my $save = new QPixmap( $buffer );
  $buffer->resize( $event->size() );
  $buffer->fill( $white );
  bitBlt( $buffer, 0, 0, $save );
}


  

  


  
