#include <qapplication.h>
#include <qmenubar.h>
#include <qmessagebox.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qpopupmenu.h>
#include <qwidget.h>

enum MenuIDs {
	COLOR_MENU_ID_BLACK,
	COLOR_MENU_ID_RED,
	COLOR_MENU_ID_BLUE,
	COLOR_MENU_ID_GREEN,
	COLOR_MENU_ID_YELLOW };

/**
  * A class that lets the user draw scribbles with the mouse. The
  * window knows how to redraw itself.
  */
class ScribbleWindow : public QWidget
{
  Q_OBJECT  // necessary because ScribbleWindow contains slots
  
public:
  ScribbleWindow();
  ~ScribbleWindow();

protected:
  virtual void mousePressEvent( QMouseEvent* );
  virtual void mouseMoveEvent( QMouseEvent* );
  virtual void paintEvent( QPaintEvent* );
  virtual void resizeEvent( QResizeEvent* );

private slots:
  void slotAbout();
  void slotAboutQt();
  void slotColorMenu( int );

private:
  QPoint _last;
  QColor _currentcolor;

  QPixmap _buffer;
  QMenuBar* _menubar;
  QPopupMenu* _filemenu;
  QPopupMenu* _colormenu;
  QPopupMenu* _helpmenu;
};

#include "qtscribble2.moc"

/** The constructor. Initializes the member variables and the menu
  * system. 
  */ 
ScribbleWindow::ScribbleWindow()
{
  // initialize member variables
  _currentcolor = black;

  // don't blank the window before repainting
  setBackgroundMode( NoBackground );

  /* The next lines build up the menubar. We first create the menus
   * one by one and add them afterwards to the menubar. */
  _filemenu = new QPopupMenu; // create a file menu
  _filemenu->insertItem( "&Quit", qApp, SLOT( quit() ) ); 
													  
  _colormenu = new QPopupMenu; // create a color menu
  _colormenu->insertItem( "B&lack", COLOR_MENU_ID_BLACK );
  _colormenu->insertItem( "&Red", COLOR_MENU_ID_RED );
  _colormenu->insertItem( "&Blue", COLOR_MENU_ID_BLUE );
  _colormenu->insertItem( "&Green", COLOR_MENU_ID_GREEN );
  _colormenu->insertItem( "&Yellow", COLOR_MENU_ID_YELLOW );
  QObject::connect( _colormenu, SIGNAL( activated( int ) ),
					this, SLOT( slotColorMenu( int ) ) );

  _helpmenu = new QPopupMenu; // create a help menu
  _helpmenu->insertItem( "&About QtScribble", this, SLOT( slotAbout() ) );
  _helpmenu->insertItem( "About &Qt", this, SLOT( slotAboutQt() ) );

  _menubar = new QMenuBar( this ); // create a menu bar
  _menubar->insertItem( "&File", _filemenu );
  _menubar->insertItem( "&Color", _colormenu );
  _menubar->insertSeparator();
  _menubar->insertItem( "&Help", _helpmenu );
}


/** 
  * The destructor. Does nothing for now.
  */
ScribbleWindow::~ScribbleWindow()
{
}


/**
  * This virtual method gets called whenever the users presses the
  * mouse over the window. It just records the position of the mouse
  * at the time of the click.
  */
void ScribbleWindow::mousePressEvent( QMouseEvent* event )
{
  _last = event->pos(); // retrieve the coordinates from the event
}


/**
  * This virtual method gets called whenever the user moves the mouse
  * while the mouse button is pressed (this is also known as
  * "dragging"). If we had called setMouseTracking( true ) before,
  * this method would also be called when the mouse was moved without
  * any button pressed. We know that we haven't, and thus don't have
  * to check whether any buttons are pressed. One caveat: If you use
  * tooltips, mouse tracking is turned on automatically.
  */
void ScribbleWindow::mouseMoveEvent( QMouseEvent* event )
{
  // create a QPainter object for drawing onto the window
  QPainter windowpainter;
  // and another QPainter object for drawing into an off-screen pixmap
  QPainter bufferpainter;

  // start painting
  windowpainter.begin( this );    // this painter paints onto the window
  bufferpainter.begin( &_buffer ); // and this one in the buffer

  // set a standard pen with the currently selected color
  windowpainter.setPen( _currentcolor );
  bufferpainter.setPen( _currentcolor );

  // draw a line in both the window and the buffer
  windowpainter.drawLine( _last, event->pos() );
  bufferpainter.drawLine( _last, event->pos() );

  // done with painting
  windowpainter.end();
  bufferpainter.end();

  // remember the current mouse position
  _last = event->pos();
}



/** 
  *	This virtual method gets called whenever the widget needs
  * painting, e.g., when it has been obscured and unhidden again.
  */
void ScribbleWindow::paintEvent( QPaintEvent* event )
{
  // copy the image from the buffer pixmap to the window
  bitBlt( this, 0, 0, &_buffer );
}


/**
  * This virtual method gets called whenever the window is resized. We
  * use it to make sure that the off-screen buffer always has the same
  * size as the window.
  * In order not to lose the original scribbling, it is first copied
  * to a temporary buffer. After the main buffer has been resized and
  * filled with white, the image is copied from the temporary buffer to
  * the main buffer.
  */
void ScribbleWindow::resizeEvent( QResizeEvent* event )
{
  QPixmap save( _buffer );
  _buffer.resize( event->size() );
  _buffer.fill( white );
  bitBlt( &_buffer, 0, 0, &save );
}


void ScribbleWindow::slotAbout()
{
  QMessageBox::information( this, "About QtScribble 2", 
							"This is the QtScribble 2 application\n"
							"Copyright 1998 by Matthias Kalle Dalheimer\n"
							);
}


void ScribbleWindow::slotAboutQt()
{
  QMessageBox::aboutQt( this, "About Qt" );
}


void ScribbleWindow::slotColorMenu( int item )
{
  switch( item )
	{
	case COLOR_MENU_ID_BLACK:
	  _currentcolor = black;
	  break;
	case COLOR_MENU_ID_RED:
	  _currentcolor = darkRed;
	  break;
	case COLOR_MENU_ID_BLUE:
	  _currentcolor = darkBlue;
	  break;
	case COLOR_MENU_ID_GREEN:
	  _currentcolor = darkGreen;
	  break;
	case COLOR_MENU_ID_YELLOW:
	  _currentcolor = yellow;
	  break;
	}
}


int main( int argc, char* argv[] )
{
  QApplication myapp( argc, argv );

  ScribbleWindow* mywidget = new ScribbleWindow;
  mywidget->setGeometry( 50, 50, 400, 400 );
  
  myapp.setMainWidget( mywidget );
  mywidget->show();
  return myapp.exec();
}
