#include <qapplication.h>
#include <qmenubar.h>
#include <qmessagebox.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qpopupmenu.h>
#include <qscrollview.h>
#include <qwidget.h>

enum MenuIDs {
	COLOR_MENU_ID_BLACK,
	COLOR_MENU_ID_RED,
	COLOR_MENU_ID_BLUE,
	COLOR_MENU_ID_GREEN,
	COLOR_MENU_ID_YELLOW };

/**
  * A class that lets the user draw scribbles with the mouse. The
  * window knows how to redraw itself.
  */
class ScribbleArea : public QWidget
{
  Q_OBJECT  // necessary because ScribbleArea contains a slot
  
public:
  ScribbleArea();
  ~ScribbleArea();

public slots:
  void setColor( QColor );

protected:
  virtual void mousePressEvent( QMouseEvent* );
  virtual void mouseMoveEvent( QMouseEvent* );
  virtual void paintEvent( QPaintEvent* );
  virtual void resizeEvent( QResizeEvent* );

private slots:
  void slotClearArea();

private:
  QPoint _last;
  QColor _currentcolor;

  QPixmap _buffer;
  QPopupMenu* _popupmenu;
};


class ScribbleWindow : public QWidget
{
  Q_OBJECT

public:
  ScribbleWindow();
  ~ScribbleWindow();

private slots:
  void slotAbout();
  void slotAboutQt();
  void slotColorMenu( int );

signals:
  void colorChanged( QColor );

protected:
  virtual void resizeEvent( QResizeEvent* );

private:
  QMenuBar* _menubar;
  QPopupMenu* _filemenu;
  QPopupMenu* _colormenu;
  QPopupMenu* _helpmenu;
  QScrollView* _scrollview;
  ScribbleArea* _scribblearea;
};


#include "qtscribble4.moc"

/** 
  * The constructor. Initializes the member variables and creates a
  * popup menu that will be popped up when the right mouse button is
  * clicked. 
  */ 
ScribbleArea::ScribbleArea()
{
  // initialize member variables
  _currentcolor = black;

  // don't blank the window before repainting
  setBackgroundMode( NoBackground );

  // create a popup menu
  _popupmenu = new QPopupMenu();
  _popupmenu->insertItem( "&Clear", this, SLOT( slotClearArea() ) );
}


/** 
  * The destructor. Deletes the popup menu.
  */
ScribbleArea::~ScribbleArea()
{
  delete _popupmenu;
}


/** 
  * This slot sets the current color for the scribble area. It will be
  * connected with the colorChanged( QColor* ) signal from the
  * ScribbleWindow.
  */
void ScribbleArea::setColor( QColor new_color )
{
  _currentcolor = new_color;
}


/**
  * This slot clears the scribble area by filling the off-screen buffer with 
  * white and copying it over to the window.
  */
void ScribbleArea::slotClearArea()
{
  // fill the off-screen buffer with plain white
  _buffer.fill( white );
  
  // and copy it over to the window
  bitBlt( this, 0, 0, &_buffer );
}


/**
  * This virtual method gets called whenever the users presses the
  * mouse over the window. When the right mouse button is pressed, it
  * pops up synchronously a previously constructed popup
  * menu. Otherwise, it just records the position of the mouse   
  * at the time of the click.
  */
void ScribbleArea::mousePressEvent( QMouseEvent* event )
{
  if( event->button() == RightButton )
	_popupmenu->exec( QCursor::pos() );
  else
	{
	  _last = event->pos(); // retrieve the coordinates from the event
	}
}


/**
  * This virtual method gets called whenever the user moves the mouse
  * while the mouse button is pressed (this is also known as
  * "dragging"). If we had called setMouseTracking( true ) before,
  * this method would also be called when the mouse was moved without
  * any button pressed. We know that we haven't, and thus don't have
  * to check whether any buttons are pressed.
  */
void ScribbleArea::mouseMoveEvent( QMouseEvent* event )
{
  // create a QPainter object for drawing onto the window
  QPainter windowpainter;
  // and another QPainter object for drawing into an off-screen pixmap
  QPainter bufferpainter;

  // start painting
  windowpainter.begin( this );    // this painter paints onto the window
  bufferpainter.begin( &_buffer ); // and this one in the buffer

  // set a standard pen with the currently selected color
  windowpainter.setPen( _currentcolor );
  bufferpainter.setPen( _currentcolor );

  // draw a line in both the window and the buffer
  windowpainter.drawLine( _last, event->pos() );
  bufferpainter.drawLine( _last, event->pos() );

  // done with painting
  windowpainter.end();
  bufferpainter.end();

  // remember the current mouse position
  _last = event->pos();
}



/** 
  *	This virtual method gets called whenever the widget needs
  * painting, e.g., when it has been obscured and unhidden again.
  */
void ScribbleArea::paintEvent( QPaintEvent* event )
{
  // copy the image from the buffer pixmap to the window
  bitBlt( this, 0, 0, &_buffer );
}


/**
  * This virtual method gets called whenever the window is resized. We
  * use it to make sure that the off-screen buffer always has the same
  * size as the window.
  * In order not to lose the original scribbling, it is first copied
  * to a temporary buffer. After the main buffer has been resized and
  * filled with white, the image is copied from the temporary buffer to
  * the main buffer.
  */
void ScribbleArea::resizeEvent( QResizeEvent* event )
{
  QPixmap save( _buffer );
  _buffer.resize( event->size() );
  _buffer.fill( white );
  bitBlt( &_buffer, 0, 0, &save );
}


ScribbleWindow::ScribbleWindow()
{
  /* The next few lines build up the menubar. We first create the menus
   * one by one and add them afterwards to the menubar. */
  _filemenu = new QPopupMenu; // create a file menu
  _filemenu->insertItem( "&Quit", qApp, SLOT( quit() ) ); 
													  
  _colormenu = new QPopupMenu; // create a color menu
  _colormenu->insertItem( "B&lack", COLOR_MENU_ID_BLACK );
  _colormenu->insertItem( "&Red", COLOR_MENU_ID_RED );
  _colormenu->insertItem( "&Blue", COLOR_MENU_ID_BLUE );
  _colormenu->insertItem( "&Green", COLOR_MENU_ID_GREEN );
  _colormenu->insertItem( "&Yellow", COLOR_MENU_ID_YELLOW );
  QObject::connect( _colormenu, SIGNAL( activated( int ) ),
					this, SLOT( slotColorMenu( int ) ) );

  _helpmenu = new QPopupMenu; // create a help menu
  _helpmenu->insertItem( "&About QtScribble", this, SLOT( slotAbout() ) );
  _helpmenu->insertItem( "About &Qt", this, SLOT( slotAboutQt() ) );

  _menubar = new QMenuBar( this ); // create a menu bar
  _menubar->insertItem( "&File", _filemenu );
  _menubar->insertItem( "&Color", _colormenu );
  _menubar->insertSeparator();
  _menubar->insertItem( "&Help", _helpmenu );

  /* We create a QScrollView and a ScribbleArea. The ScribbleArea will
   * be managed by the scroll view.*/
  _scrollview = new QScrollView( this );
  _scrollview->setGeometry( 0, _menubar->height(), 
						   width(), height() - _menubar->height() );
  _scribblearea = new ScribbleArea();
  _scribblearea->setGeometry( 0, 0, 1000, 1000 );
  _scrollview->addChild( _scribblearea );
  QObject::connect( this, SIGNAL( colorChanged( QColor ) ),
                    _scribblearea, SLOT( setColor( QColor ) ) );
}


ScribbleWindow::~ScribbleWindow()
{
}


void ScribbleWindow::resizeEvent( QResizeEvent* event )
{
  /* When the whole window is resized, we have to rearrange the geometry
   * in the ScribbleWindow as well. Note that the ScribbleArea does not need 
   * to be changed. */
  _scrollview->setGeometry( 0, _menubar->height(), 
						   width(), height() - _menubar->height() );
}
  

void ScribbleWindow::slotAbout()
{
  QMessageBox::information( this, "About QtScribble 4", 
							"This is the QtScribble 4 application\n"
							"(C) 1998 by Matthias Kalle Dalheimer\n"
							);
}


void ScribbleWindow::slotAboutQt()
{
  QMessageBox::aboutQt( this, "About Qt" );
}


void ScribbleWindow::slotColorMenu( int item )
{
  switch( item )
	{
	case COLOR_MENU_ID_BLACK:
		emit colorChanged( black );
		break;
	case COLOR_MENU_ID_RED:
		emit colorChanged( darkRed );
		break;
	case COLOR_MENU_ID_BLUE:
		emit colorChanged( darkBlue );
		break;
	case COLOR_MENU_ID_GREEN:
		emit colorChanged( darkGreen );
		break;
	case COLOR_MENU_ID_YELLOW:
		emit colorChanged( yellow );
		break;
	}
}



int main( int argc, char* argv[] )
{
  QApplication myapp( argc, argv );

  ScribbleWindow* mywidget = new ScribbleWindow();
  mywidget->setGeometry( 50, 50, 400, 400 );
 
  myapp.setMainWidget( mywidget );
  mywidget->show();
  return myapp.exec();
}
