#include <qapplication.h>
#include <qlineedit.h>
#include <qvalidator.h>
#include <ctype.h>

/** 
  *	A class that validates ISBN numbers. See text for the
  * specification.
  */
class ISBNValidator : public QValidator
{
public:
  ISBNValidator() : QValidator( 0 ) {};
  virtual State validate( QString&, int& );
  virtual void fixup( QString& );
};


/**
  * This method is called after every input. Since we can check every
  * character as soon as it is entered, we only need to check one
  * character at a time for validity. In order to determine whether
  * the whole string is acceptable, we must of course look at the whole
  * string.
  * Note that input can also be deleted. This makes it faily useless
  * to save state (like the number of entered digits) between
  * invocations of validate(), because we would have to keep an extra
  * copy of the last string around.
  */
QValidator::State ISBNValidator::validate( QString& text, int& pos )
{
  /* An empty string could be made acceptable */
  if( !text.length() )
	return Valid;

  /* Protect against spurious calles to validate() */
  if( (unsigned int)pos >= text.length() )
	return Valid;

  /* Anything but decimal digits and dashes is invalid. We only need
   * to check the character at the cursor positions, this speeds
   * things up massively.
   */
  if( !isdigit( text[pos] ) && ( text[pos] != '-' ) )
	{
	  debug( "Typed character is neither digit nor dash\n" );
	  return Invalid;
	}

  /* Dashes are only valid at positions 1 (the second position) and
   * position 8 (if there was no dash at position 1) or position 9 (if
   * there was a dash at position 1).
   */
  if( text[pos] == '-' )
	{
	  if( pos != 1 && pos != 8 && pos != 9 )
		{
		  debug( "Dash at wrong position\n" );
		  return Invalid;
		}
	  
	  if( ( pos == 8 && text[1] == '-' ) || 
		  ( pos == 9 && text[1] != '-' ) )
		{
		  debug( "Dash at wrong position\n" );
		  return Invalid;
		}
	}
  
  /* If the characters entered so far are valid, but the string
   * contains less than ten digits, it could be made acceptable, but
   * is not yet.
   */
  int numdigits = text.length();
  if( text.length() > 1 && text[1] == '-' ) numdigits--;
  if( text.length() > 8 && text[8] == '-' ) numdigits--;
  if( text.length() > 9 && text[9] == '-' ) numdigits--;

  if( numdigits < 10 )
	{
	  debug( "Less than ten digits; input valid but not yet acceptable\n" );
	  return Valid;
	}

  /* More than ten digits are always invalid */
  if( numdigits > 10 )
	{
	  debug( "More than ten digits; input invalid\n" );
	  return Invalid;
	}

  /* If we have exactly ten digits, but the dashes are not in place,
   * we don't declare the input Acceptable, but Valid only. In this
   * case, the text entry field will call fixup() where we can add the
   * missing dashes.
   */
  if( text[1] != '-' || text[9] != '-' )
	{
	  debug( "Ten digits, but dashes not in the right places. Could be fixed up\n" );
		return Valid;
	}

  /* Everything is as it should be. */
  debug( "Input acceptable\n" ); 
  return Acceptable;
}


/**
  * This method is called when the user has pressed return, but
  * validate() has judged the string valid, but not acceptable. Note
  * that fixup() can do all it must to help but is not required to
  * return an acceptable string. It is guaranteed that the caller will
  * validate() the string once again after the call to fixup().
  */
void ISBNValidator::fixup( QString& text )
{
  /* We can only fix the input up if only the dashes are missing,
   * since we cannot implement the missing digits. (We could at least
   * compute the last one with the checksum algorithm, but won't do so
   * here.)
   */

  /* If at least one digit has been entered, but there is no dash at
   * position 1, insert one.
   */
  if( text.length() && text[1] != '-' )
	text.insert( 1, '-' );

  /* If at least nine digits have been entered, but there is no dash
   * at position 10 (because of the last two lines, we can safely
   * assume that the first dash is already in place), insert one.
   */
  if( text.length() >= 10 /* nine digits plus dash */ && text[10] != '-' )
	text.insert( 10, '-' );
}


/**
  * This class only servers as a receiver for the returnPressed()
  * signal so that we can know when the input has been accepted.
  */
class ReturnReceiver : public QObject
{
  Q_OBJECT

public slots:
  void slotReturnPressed();
};


void ReturnReceiver::slotReturnPressed()
{
  /* When this string appears, the input was acceptable. */
  debug( "return pressed - input accepted\n" ); 
}

#include "validator.moc"

int main( int argc, char* argv[] )
{
  QApplication myapp( argc, argv );
  
  // create a line edit
  QLineEdit* myedit = new QLineEdit();
  myedit->resize( 100, 30 );

  // create and assign a validator for the line edit
  ISBNValidator* myvalidator = new ISBNValidator();
  myedit->setValidator( myvalidator );

  // setup a receiver for the returnPressed() signal
  ReturnReceiver receiver;
  QObject::connect( myedit, SIGNAL( returnPressed() ),
					&receiver, SLOT( slotReturnPressed() ) );
  
  myapp.setMainWidget( myedit );
  myedit->show();
  return myapp.exec();
}
